<?php
/*
Plugin Name: Search Engine query in Wordpress
Plugin URI: http://www.francesco-castaldo.com/plugins-and-widgets/search-engine-query-in-wordpress/
Description: If the visitor comes from a known search engine, the widget grabs the used search query and shows internal blog posts that match that query.
Author: Francesco Castaldo
Version: 1.1
Author URI: http://www.francesco-castaldo.com/

Installing
1. Unzip and upload 'search-engine-query-in-wp' directory to your '/wp-content/plugins/' directory
2. Activate the plugin through the 'Plugins' menu in the Admin panel
3. Activete the widget in the 'Appearance -> Widgets' section of the Admin panel
4. Edit widget settings for fine tuning (not mandatory)

Changelog
1.1 = More search engines supported
*/

/*
Copyright 2009  Francesco Castaldo  (email : fcastaldo@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * If the page is a single post, it returns the first category the post is in
 */
function SEQ_getPostCategory() {
	if (is_single()) {
		foreach((get_the_category()) as $category) {
			return $category;
			break;
		}
	}
	return null;
}

/**
 * Default values for the widget
 */
function SEQ_getDefaults() {
	$options['title'] = 'Related Posts';
	$options['show_category'] = 'true';
	$options['track_clicks'] = 'true';
	$options['number'] = 5;
	return $options;
}

/**
 * Widget box
 */
function SEQ_in_wp($args) {

	//global $user_level;
	extract($args);

	$options = get_option('seq_in_wp');

	if( $options == false ) {
		$options = SEQ_getDefaults();
	}

	$title = $options['title'];
	$moreLink = null;

	$searchQuery = SEQ_getSearchEngineQueryString();

	$related = null;
	if ($searchQuery != null) {
		$related = new WP_Query("s=".$searchQuery[1]."&showposts=".$options['number']);
		if ($related->have_posts()) {
			$widget_text = "<p>You were looking for <strong>\"".$searchQuery[1]."\"</strong> on <em>".$searchQuery[0]."</em>. You might be interested in:</p>";
			$moreLink = "/?s=".$searchQuery[1];
			$campaignParams = "?utm_source=search&utm_medium=link&utm_campaign=related+posts+widget&utm_term=".urlencode($searchQuery[1]);
		}
	}

	if ($related == null && $options['show_category'] == "true") {
		// the search with google keywords did not returned any post. Let's invite the user to read the ones in the same category of the main post

		$post_category = SEQ_getPostCategory();

		if ($post_category != null) {
			$widget_text = "<p>In the <strong>".$post_category->cat_name."</strong> category:</p>";
			$related = new WP_Query("cat=".$post_category->cat_ID."&showposts=5&orderby=date");
			$moreLink = get_category_link($post_category->cat_ID);
			$campaignParams = "?utm_source=category&utm_medium=link&utm_campaign=Search+engine+query+in+WP+widget&utm_term=".urlencode($post_category->cat_name);
		}
	}

	if ($options['track_clicks'] != "true") {
		$campaignParams = "";
	}

	if ($related != null && $related->have_posts()) {
		echo $before_widget;
		echo $before_title . $title . $after_title;
?>
<div class="relatedArticles">
	<?php echo($widget_text); ?>
	<ul>
		<?php while ($related->have_posts()) : $related->the_post(); ?>
			<li><a href="<?php the_permalink(); ?><?php echo($campaignParams); ?>"><?php the_title(); ?></a></li>
		<?php endwhile; ?>
	</ul>
	<?php if ($moreLink != null) { ?>
	<p class="more"><a href="<?php echo($moreLink); ?><?php echo($campaignParams); ?>">more...</a></p>
	<?php } ?>
</div>
<?php
			echo $after_widget;
	} else {
			//nothing found
	}
}

/**
 * Widget init function
 */
function SEQ_in_wp_widget_init() {
	if (function_exists("register_widget_control")) {
		register_widget_control('Search Engine query in WP', 'SEQ_in_wp_widget_control', 500, 250);
	}
	if (function_exists("register_sidebar_widget")) {
		register_sidebar_widget('Search Engine query in WP', 'SEQ_in_wp');
	}
}

/**
 * Adds the widget stylesheet in the <head> section
 */
function SEQ_in_wp_widget_HeadAction()
{
	echo '<link rel="stylesheet" href="'.trailingslashit( get_bloginfo('wpurl') ).PLUGINDIR.'/'. dirname( plugin_basename(__FILE__) ).'/seq_in_wp.css" type="text/css" />';
}

if (function_exists("add_action")) {
	add_action("init", "seq_in_wp_widget_init");
	add_action('wp_head', 'SEQ_in_wp_widget_HeadAction');
}

/**
 * Widget control panel
 */
function SEQ_in_wp_widget_control() {
	$options = $newoptions = get_option('seq_in_wp');

	if ($options == false) {
		$newoptions = SEQ_getDefaults();
	}

	if (isSet($_POST["seq_in_wp-submit"])) {
		$newoptions['title'] = strip_tags(stripslashes($_POST["seq_in_wp-title"]));
		$newoptions['number'] = strip_tags(stripslashes($_POST["seq_in_wp-number"]));
		if (!is_numeric($newoptions['number']))	{
			$newoptions['number'] = $options['number'];
		} else {
			if ($newoptions['number'] > 10 || $newoptions['number'] < 1) {
				$newoptions['number'] = $options['number'];
			}
		}
		$newoptions['show_category'] = strip_tags(stripslashes($_POST["seq_in_wp-show_category"]));
		$newoptions['track_clicks'] = strip_tags(stripslashes($_POST["seq_in_wp-track_clicks"]));
	}
	if ($options != $newoptions) {
		$options = $newoptions;
		update_option('seq_in_wp', $options);
	}

	$title = wp_specialchars($options['title']);
	$show_category = wp_specialchars($options['show_category']);
	$stats_tracking = wp_specialchars($options['track_clicks']);
	$number =  wp_specialchars($options['number']);
?>
	<p><label for="seq_in_wp-title"><?php _e('Widget title:'); ?> <input style="width: 250px;" id="seq_in_wp-title" name="seq_in_wp-title" type="text" value="<?php echo $title; ?>" /></label></p>
	<p><label for="seq_in_wp-number"><?php _e('Number of results (numeric value between 1 and 10):'); ?> <input style="width: 40px;" id="seq_in_wp-number" name="seq_in_wp-number" type="text" value="<?php echo $number; ?>" /></label></p>
	<p><label for="seq_in_wp-show_category"><?php _e('Show category posts if no results with "search engine query" are found:'); ?> <input id="seq_in_wp-show_category" name="seq_in_wp-show_category" type="checkbox" value="true" <?php if ($show_category == "true") { echo "checked=\"checked\""; } ?>" /></label></p>
	<p><label for="seq_in_wp-track_clicks"><?php _e('Track clicks'); ?> <input id="seq_in_wp-track_clicks" name="seq_in_wp-track_clicks" type="checkbox" value="true" <?php if ($stats_tracking == "true") { echo "checked=\"checked\""; } ?>" /></label></p>
	<p><?php _e('If you choose to track clicks on the widget\'s links, the plugin will add to internal links some special parameters that will allow you to understand how much the box is used and in which manner. In your analytics tool you\'ll find a "Search engine query in WP widget" campain with all the data.'); ?> <a href="http://www.francesco-castaldo.com/plugins/search-engine-query-in-wordpress-related-content/" target="_blank"><?php _e('More information here'); ?></a></p>
	<p><a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=2797439" rel="nofollow" target="_blank" title="<?php _e('If this plugin is worth for you, consider donating a very few bucks for the time I spent developing it ;)'); ?>"><img src="https://www.paypal.com/en_US/i/btn/btn_donateCC_LG_global.gif" alt="<?php _e('If this plugin is worth for you, consider donating a very few bucks for the time I spent on it ;)'); ?>" /></a>
	<input type="hidden" id="seq_in_wp-submit" name="seq_in_wp-submit" value="1" />
<?php
}

/**
 * Get the search engine query from the referer server variable, if available
 * @return a 2 elements array: in first position the search engine name, in second position the search engine query
 */
function SEQ_getSearchEngineQueryString()
{
	if (isSet($_SERVER["HTTP_REFERER"]) && strlen($_SERVER['HTTP_REFERER']) > 5) {
		$referral = $_SERVER["HTTP_REFERER"];
	} else {
		return null;
	}

	if (strpos($referral, "google.") != false) {
		$engine = "Google";
	} elseif (strpos($referral, "yahoo.") != false)	{
		$engine = "Yahoo!";
	} elseif (strpos($referral, "msn.") != false) {
		$engine = "Msn";
	} elseif (strpos($referral, "ask.") != false) {
		$engine = "Ask";
	} elseif (strpos($referral, "live.") != false) {
		$engine = "Live Search";
	} elseif (strpos($referral, "lycos.") != false) {
		$engine = "Lycos";
	} elseif (strpos($referral, "aol.") != false) {
		$engine = "Aol";
	} elseif (strpos($referral, "dmoz.") != false) {
		$engine = "Dmoz";
	} elseif (strpos($referral, "altavista.") != false) {
		$engine = "Altavista";
	} elseif (strpos($referral, "gigablast.") != false) {
		$engine = "Gigablast";
	} elseif (strpos($referral, "alltheweb.") != false) {
		$engine = "AllTheWeb";
	} elseif (strpos($referral, "hotbot.") != false) {
		$engine = "HotBot";
		
	} else {
		// unknown search engine
		return null;
	}

	$token = null;

	if ($engine == "Google") {
		// ricerco il parametro usato per indicare la query cercata
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		} elseif (strpos($referral, "&as_q=") != false) {
			$token = "&as_q";
		} elseif (strpos($referral, "?as_q=") != false) {
			$token = "?as_q=";
		} elseif (strpos($referral, "&as_epq=") != false) {
			$token = "&as_epq=";
		} elseif (strpos($referral, "?as_epq=") != false) {
			$token = "?as_epq=";
		} elseif (strpos($referral, "&as_oq=") != false) {
			$token = "&as_oq=";
		} elseif (strpos($referral, "?as_oq=") != false) {
			$token = "?as_oq=";
		}
	} elseIf ($engine == "Yahoo!") {
		if (strpos($referral, "&p=") != false) {
			$token = "&p=";
		} elseif (strpos($referral, "?p=") != false) {
			$token = "?p=";
		}
	} elseIf ($engine == "Msn") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "Ask") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "Live Search") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "Lycos") {
		if (strpos($referral, "&query=") != false) {
			$token = "&query=";
		} elseif (strpos($referral, "?query=") != false) {
			$token = "?query=";
		}
	} elseIf ($engine == "Aol") {
		if (strpos($referral, "&query=") != false) {
			$token = "&query=";
		} elseif (strpos($referral, "?query=") != false) {
			$token = "?query=";
		}
	} elseIf ($engine == "Dmoz") {
		if (strpos($referral, "&search=") != false) {
			$token = "&search=";
		} elseif (strpos($referral, "?search=") != false) {
			$token = "?search=";
		}
	} elseIf ($engine == "Altavista") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "Gigablast") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "AllTheWeb") {
		if (strpos($referral, "&q=") != false) {
			$token = "&q=";
		} elseif (strpos($referral, "?q=") != false) {
			$token = "?q=";
		}
	} elseIf ($engine == "HotBot") {
		if (strpos($referral, "&query=") != false) {
			$token = "&query=";
		} elseif (strpos($referral, "?query=") != false) {
			$token = "?query=";
		}
	}

	if ($token == null) {
		return null;
	}

	$nextEndPos = strpos($referral, "&", strpos($referral, $token)+2);
	if ($nextEndPos == false) {
		// la query di ricerca  l'ultimo parametro
		$searchPhraseLen = strlen($referral) - strpos($referral, $token);
	} else {
		$searchPhraseLen = $nextEndPos - strpos($referral, $token) - strlen($token);
	}

	$searchPhrase = substr($referral, (strlen($token) + strpos($referral, $token)), $searchPhraseLen);
	$searchPhrase = trim(urldecode(strtolower($searchPhrase)));

	//elimino dalla search phrase per le ricerche con comandi speciali (site: link: cache: info: related: )

	$cmd = null;
	if (strpos($searchPhrase, "site:") !== false) {
		$cmd = "site:";
	}
	if (strpos($searchPhrase, "link:") !== false) {
		$cmd = "link:";
	}
	if (strpos($searchPhrase, "cache:") !== false) {
		$cmd = "cache:";
	}
	if (strpos($searchPhrase, "info:") !== false) {
		$cmd = "info:";
	}
	if (strpos($searchPhrase, "related:") !== false) {
		$cmd = "related:";
	}

	// ora devo estrarre il comando dalla searchPhrase
	if ($cmd != null) {
		$cmdStart = strpos($searchPhrase, $cmd);
		$nextSpace = strpos($searchPhrase, " ", (strpos($searchPhrase, $cmd) + 1));
		if ($nextSpace == false) {
			// il comando  alla fine
			$nextSpace = strlen($searchPhrase);
		}
		$cmd = substr($searchPhrase, $cmdStart, $nextSpace);
		$searchPhrase = trim(str_replace($cmd, "", $searchPhrase));
	}

	// elimino le virgolette che nella nostra ricerca non sono significative
	$searchPhrase = str_replace("\"", "", $searchPhrase);

	$searchPhrase = str_replace("'", "", $searchPhrase);

	// se l'utente ha inserito + come carattere di ricerca, non viene normalizzato dall'url decode
	$searchPhrase = str_replace("+", " ", $searchPhrase);

	// se la frase cercata  pi corta di tre caratteri, non  interessante
	if (strlen($searchPhrase) < 3) {
		return null;
	}

	return array($engine, $searchPhrase);
}

?>